#include "VBFNLOConfig.h"
/*  Written in 2016 by David Blackman and Sebastiano Vigna (vigna@acm.org)

To the extent possible under law, the author has dedicated all copyright
and related and neighboring rights to this software to the public domain
worldwide. This software is distributed without any warranty.

See <http://creativecommons.org/publicdomain/zero/1.0/>. */

#include <stdint.h>
#include <stdio.h>

/* This is the successor to xorshift128+. It is the fastest full-period
   generator passing BigCrush without systematic failures, but due to the
   relatively short period it is acceptable only for applications with a
   mild amount of parallelism; otherwise, use a xorshift1024* generator.

   Beside passing BigCrush, this generator passes the PractRand test suite
   up to (and included) 16TB, with the exception of binary rank tests,
   which fail due to the lowest bit being an LFSR; all other bits pass all
   tests. We suggest to use a sign test to extract a random Boolean value.
   
   Note that the generator uses a simulated rotate operation, which most C
   compilers will turn into a single instruction. In Java, you can use
   Long.rotateLeft(). In languages that do not make low-level rotation
   instructions accessible xorshift128+ could be faster.

   The state must be seeded so that it is not everywhere zero. If you have
   a 64-bit seed, we suggest to seed a splitmix64 generator and use its
   output to fill s. */

#ifdef __cplusplus
extern "C" 
#endif
/* void FC_FUNC_(xorshift_next,XORSHIFT_NEXT)(void); */
/* void FC_FUNC_(xorshift_seed,XORSHIFT_SEED)(long* seed1, long* seed2); */
/* #ifdef __cplusplus */
/* } */
/* #endif */

/* void FC_FUNC_(xorshift_getrand, XORSHIFT_GETRAND)(double* rand); */

uint64_t s[2];
int ndimen;

uint64_t splitmix64_next(uint64_t seed); 

static inline uint64_t rotl(const uint64_t x, int k) {
	return (x << k) | (x >> (64 - k));
}

/* external seed function; will get numbers with many bits 0
 * initialize seed using splitmix64 as suggested by original author */
void FC_FUNC_(xorshift_seed,XORSHIFT_SEED)(int *ndim, int *seed) {
    ndimen = *ndim;
    uint64_t useed = (uint64_t) *seed;
    s[0] = splitmix64_next(useed);
    s[1] = splitmix64_next(s[0]);
    /* printf("Seeded with %i, dim %i, then %uli, %uli", *seed, ndimen, s[0], s[1]); */
}

uint64_t xorshift_next(void) {
	const uint64_t s0 = s[0];
	uint64_t s1 = s[1];
	const uint64_t result = s0 + s1;

	s1 ^= s0;
	s[0] = rotl(s0, 55) ^ s1 ^ (s1 << 14); // a, b
	s[1] = rotl(s1, 36); // c

	return result;
}

/* use xorshift to return array of ndim double values */
void FC_FUNC_(xorshift_getrand,XORSHIFT_GETRAND)(double* rand) {
    for(int i = 0; i < ndimen; i++) {
        rand[i] = xorshift_next()/(double)UINT64_MAX;
    }
}

/* This is the jump function for the generator. It is equivalent
   to 2^64 calls to next(); it can be used to generate 2^64
   non-overlapping subsequences for parallel computations. */

void FC_FUNC_(xorshift_jump,XORSHIFT_JUMP)(void) {
	static const uint64_t JUMP[] = { 0xbeac0467eba5facb, 0xd86b048b86aa9922 };

	uint64_t s0 = 0;
	uint64_t s1 = 0;
	for(int i = 0; i < 2; i++)
		for(int b = 0; b < 64; b++) {
			if (JUMP[i] & 1ULL << b) {
				s0 ^= s[0];
				s1 ^= s[1];
			}
			xorshift_next();
		}

	s[0] = s0;
	s[1] = s1;
}
